@@ -38,9 +38,13 @@
 
 import abc
 import datetime
+from abc import ABC
+from html.parser import HTMLParser
+from math import sqrt
 import os
 import shutil
 import sys
+
 import yaml
 
 from arg.Common.argInformationObject import argInformationObject
@@ -91,8 +95,7 @@ class argBackendBase:
         """
 
         # Return value of instance variable
-        return argMultiFontStringHelper.Types.get(
-            "BackendTypes", {}).get(self.Type, {}).get("captions", '')
+        return argMultiFontStringHelper.Types.get("BackendTypes", {}).get(self.Type, {}).get("captions", '')
 
     @staticmethod
     def get_timestamp():
@@ -264,15 +267,15 @@ class argBackendBase:
                 body_dict[key] = [m.get(c) for c in other_cols]
 
             caption_string = argMultiFontStringHelper(self)
-            caption_string.append("Meta-information of ", "default")
+            caption_string.append("Meta-information of ", 0)
             data_extension = item.get("parameters", {}).get("extension")
             if data_extension:
-                caption_string.append(" all files with extension ", "default")
-                caption_string.append(data_extension, "typewriter")
-                caption_string.append(" in data directory.", "default")
+                caption_string.append(" all files with extension ", 0)
+                caption_string.append(data_extension, 4)
+                caption_string.append(" in data directory.", 0)
             else:
-                caption_string.append(data_set, "typewriter")
-            caption_string.append(".", "default")
+                caption_string.append(data_set, 4)
+            caption_string.append(".", 0)
             self.add_table(
                 header_list,
                 body_dict,
@@ -285,8 +288,8 @@ class argBackendBase:
             for m in meta_info:
                 # Create caption string
                 caption_string = argMultiFontStringHelper(self)
-                caption_string.append("Meta-information of ", "default")
-                caption_string.append(data_set, "typewriter")
+                caption_string.append("Meta-information of ", 0)
+                caption_string.append(data_set, 4)
 
                 # Generate meta-information table
                 self.add_table(
@@ -306,8 +309,8 @@ class argBackendBase:
             # Create table of topological properties
             header_list, body_dict = data.summarize(type="topology")
             caption_string = argMultiFontStringHelper(self)
-            caption_string.append("Topological properties of ", "default")
-            caption_string.append(file_name, "typewriter")
+            caption_string.append("Topological properties of ", 0)
+            caption_string.append(file_name, 4)
             self.add_table(header_list, body_dict, caption_string, True)
 
             # Create table of mesh blocks if requested and at least one exists
@@ -315,8 +318,8 @@ class argBackendBase:
                 header_list, body_dict = data.summarize(type="blocks")
                 if header_list:
                     caption_string.clear()
-                    caption_string.append("Element blocks of ", "default")
-                    caption_string.append(file_name, "typewriter")
+                    caption_string.append("Element blocks of ", 0)
+                    caption_string.append(file_name, 4)
                     self.add_table(header_list, body_dict, caption_string, True)
 
             # Create table of node and side sets if requested and at least one exists
@@ -325,17 +328,17 @@ class argBackendBase:
                     header_list, body_dict = data.summarize(type="sets", set_type=set_type)
                     if header_list:
                         caption_string.clear()
-                        caption_string.append(set_type.title(), "default")
-                        caption_string.append(" sets of ", "default")
-                        caption_string.append(file_name, "typewriter")
+                        caption_string.append(set_type.title(), 0)
+                        caption_string.append(" sets of ", 0)
+                        caption_string.append(file_name, 4)
                         self.add_table(header_list, body_dict, caption_string, True)
 
             # Create table of variables if some are present
             header_list, body_list = data.summarize(type="variable")
             if body_list:
                 caption_string.clear()
-                caption_string.append("Variables of ", "default")
-                caption_string.append(file_name, "typewriter")
+                caption_string.append("Variables of ", 0)
+                caption_string.append(file_name, 4)
                 self.add_table(header_list, body_list, caption_string, True)
 
     def add_information(self, item):
@@ -368,18 +371,18 @@ class argBackendBase:
         if not prop_info:
             # Report missing values for property type
             multi_font_string = argMultiFontStringHelper(self)
-            multi_font_string.append("No values for ", "default")
+            multi_font_string.append("No values for ", 0)
             p_items = False
             for p in prop_items:
                 # Report missing values for property items
                 if p_items:
-                    multi_font_string.append(", ", "default")
+                    multi_font_string.append(", ", 0)
                 p_items = True
-                multi_font_string.append(p, "typewriter")
+                multi_font_string.append(p, 4)
             if p_items:
-                multi_font_string.append(" items of ", "default")
-            multi_font_string.append(prop_type, "typewriter")
-            multi_font_string.append(" property were found.", "default")
+                multi_font_string.append(" items of ", 0)
+            multi_font_string.append(prop_type, 4)
+            multi_font_string.append(" property were found.", 0)
             self.add_paragraph({"string": multi_font_string})
 
             # Do not add anything else when values are missing
@@ -396,20 +399,20 @@ class argBackendBase:
                 tab_head = []
                 for p_name in prop_info.get_names():
                     multi_font_string = argMultiFontStringHelper(self)
-                    multi_font_string.append(p_name, "typewriter")
+                    multi_font_string.append(p_name, 4)
                     tab_head.append(multi_font_string)
 
                 # Iterate over property object items
                 for info_key, info_value in prop_info.iterator():
                     multi_font_string = argMultiFontStringHelper(self)
                     if prop_type:
-                        multi_font_string.append("Values of ", "default")
-                        multi_font_string.append(prop_type, "typewriter")
-                        multi_font_string.append(" property for ", "default")
+                        multi_font_string.append("Values of ", 0)
+                        multi_font_string.append(prop_type, 4)
+                        multi_font_string.append(" property for ", 0)
                     else:
-                        multi_font_string.append("Values for ", "default")
-                    multi_font_string.append(info_key, "typewriter")
-                    multi_font_string.append(".", "default")
+                        multi_font_string.append("Values for ", 0)
+                    multi_font_string.append(info_key, 4)
+                    multi_font_string.append(".", 0)
                     self.add_table(
                         tab_head,
                         info_value,
@@ -426,15 +429,15 @@ class argBackendBase:
         elif prop_items and isinstance(prop_info, dict):
             # Create table for property items only if needed
             tab_head = [argMultiFontStringHelper(self)]
-            tab_head[0].append(prop_type, "typewriter")
+            tab_head[0].append(prop_type, 4)
             tab_head += [argMultiFontStringHelper(self)
                          for _ in prop_items]
             for p in prop_items:
                 tab_head.append(p)
             multi_font_string = argMultiFontStringHelper(self)
-            multi_font_string.append("Values of ", "default")
-            multi_font_string.append(prop_type, "typewriter")
-            multi_font_string.append(" properties.", "default")
+            multi_font_string.append("Values of ", 0)
+            multi_font_string.append(prop_type, 4)
+            multi_font_string.append(" properties.", 0)
             self.add_table(
                 tab_head,
                 dict(prop_info),
@@ -456,14 +459,14 @@ class argBackendBase:
                     if prop_item and prop_values and isinstance(prop_values, dict):
                         tab_head = [argMultiFontStringHelper(self),
                                     argMultiFontStringHelper(self)]
-                        tab_head[0].append(prop_type, "typewriter")
-                        tab_head[1].append(prop_item, "typewriter")
+                        tab_head[0].append(prop_type, 4)
+                        tab_head[1].append(prop_item, 4)
                         multi_font_string = argMultiFontStringHelper(self)
-                        multi_font_string.append("Values of ", "default")
-                        multi_font_string.append(prop_item, "typewriter")
-                        multi_font_string.append(" for property ", "default")
-                        multi_font_string.append(prop_type, "typewriter")
-                        multi_font_string.append('.', "default")
+                        multi_font_string.append("Values of ", 0)
+                        multi_font_string.append(prop_item, 4)
+                        multi_font_string.append(" for property ", 0)
+                        multi_font_string.append(prop_type, 4)
+                        multi_font_string.append('.', 0)
                         self.add_table(
                             tab_head,
                             list(prop_values.items()),
@@ -578,3 +581,192 @@ class argBackendBase:
 
     _CELL_PROPS = {0: 'value', 1: 'background-color', 2: 'foreground-color', 3: 'horizontal-alignment',
                    4: 'vertical-alignment'}
+
+    @staticmethod
+    def nesting_html_list(html_list: list) -> list:
+        """
+        Function takes html_list which is a representation of HTML document and returns nested list which corresponds to
+        document structure.
+        """
+        nested_list = list()
+        current_list = list()
+        nested_active = 0
+        for pos in html_list:
+            if pos[0].startswith('_!') and pos[0].endswith('!_'):
+                if nested_active == 0:
+                    current_list = list()
+                nested_active += 1
+                if pos[1]:
+                    current_list.append(pos[0][2:-2])
+                    current_list.append({'attrs': pos[1]})
+                else:
+                    current_list.append(pos[0][2:-2])
+            elif pos[0].startswith('!__') and pos[0].endswith('__!'):
+                current_list.append({pos[0][3:-3]: pos[1]})
+            elif pos[0].startswith('__!') and pos[0].endswith('!__'):
+                nested_active -= 1
+                if nested_active == 0:
+                    nested_list.append(current_list)
+
+        return nested_list
+
+    def map_nested_list_to_amfsh(self, nested_list: list) -> list:
+        """ Iterates over nested list and returns a final list which consists information needed to be put into Word
+            document.
+        """
+        returned_list = list()
+        tags = {'h1': {'font': 9}, 'h2': {'font': 10}, 'h3': {'font': 11}, 'h4': {'font': 12}, 'h5': {'font': 13},
+                'h6': {'font': 14}}
+        for html_tag in nested_list:
+            amfsh = argMultiFontStringHelper()
+            alignment = 'LEFT'
+            indent = None
+            attrs = None
+            string = ''
+            # Support case for headings from h1 to h6
+            if html_tag[0] in tags.keys():
+                if len(html_tag) == 2 and html_tag[1].get('data', None) is not None:
+                    string = html_tag[1].get('data', None)
+                    attrs = None
+                elif len(html_tag) == 3:
+                    if html_tag[1].get('data', None) is None:
+                        string = html_tag[2].get('data', None)
+                        attrs = html_tag[1].get('attrs', None)
+                color = None
+                highlight_color = None
+                if attrs is not None:
+                    attrs = self.decode_attrs(attrs=attrs)
+                    color = attrs.get('color', None)
+                    highlight_color = attrs.get('highlight_color', None)
+                    if attrs.get('alignment', None) is not None:
+                        alignment = attrs.get('alignment', None)
+                    if attrs.get('margin-left', None) is not None:
+                        indent = ['margin-left', attrs.get('margin-left', None)]
+                    elif attrs.get('margin-right', None) is not None:
+                        indent = ['margin-right', attrs.get('margin-right', None)]
+                amfsh.append(string=string, font=tags.get(html_tag[0]).get('font'), color=color,
+                             highlight_color=highlight_color)
+                returned_list.append((alignment, amfsh, indent))
+            # Support case for Paragraph
+            elif html_tag[0] == 'p':
+                font = 0
+                color = None
+                highlight_color = None
+                attrs = None
+                for elem in html_tag:
+                    if isinstance(elem, str):
+                        font_map = {'p': 0, 'u': 3, 'strong': 2, 'em': 1, 's': 5, 'span': 0, 'sub': 6}
+                        font = font_map.get(elem, 0)
+                    elif isinstance(elem, dict) and elem.get('attrs', None) is not None:
+                        attrs = self.decode_attrs(attrs=elem.get('attrs'))
+                        color = attrs.get('color', None)
+                        highlight_color = attrs.get('highlight_color', None)
+                        if attrs.get('alignment', None) is not None:
+                            alignment = attrs.get('alignment', None)
+                        if attrs.get('margin-left', None) is not None:
+                            indent = ['margin-left', attrs.get('margin-left', None)]
+                        elif attrs.get('margin-right', None) is not None:
+                            indent = ['margin-right', attrs.get('margin-right', None)]
+                    elif isinstance(elem, dict) and elem.get('data', None) is not None:
+                        string = elem.get('data')
+                        amfsh.append(string=string, font=font, color=color, highlight_color=highlight_color)
+                        font = 0
+                        color = None
+                        highlight_color = None
+                        attrs = None
+                returned_list.append((alignment, amfsh, indent))
+            # Supports case for Ordered/Unordered lists
+            elif html_tag[0] == 'ul' or html_tag[0] == 'ol':
+                list_list = list()
+                list_type = None
+                cur_list_word = None
+                font = 0
+                for elem in html_tag:
+                    if isinstance(elem, str) and bool(elem == 'ul' or elem == 'ol'):
+                        list_map = {'ul': 'List Bullet', 'ol': 'List Number'}
+                        list_type = list_map.get(elem, None)
+                    elif isinstance(elem, str) and elem == 'li':
+                        cur_list_word = list()
+                    elif isinstance(elem, dict) and elem.get('attrs', None) is not None:
+                        attrs = self.decode_attrs(attrs=elem.get('attrs'))
+                        color = attrs.get('color', None)
+                        highlight_color = attrs.get('highlight_color', None)
+                        if attrs.get('alignment', None) is not None:
+                            alignment = attrs.get('alignment', None)
+                        if attrs.get('margin-left', None) is not None:
+                            indent = ['margin-left', attrs.get('margin-left', None)]
+                        elif attrs.get('margin-right', None) is not None:
+                            indent = ['margin-right', attrs.get('margin-right', None)]
+                    elif isinstance(elem, dict) and elem.get('data', None) is not None:
+                        string = elem.get('data')
+                        amfsh = argMultiFontStringHelper()
+                        amfsh.append(string=string, font=font, color=color, highlight_color=highlight_color)
+                        cur_list_word = [alignment, amfsh, indent, list_type]
+                        list_list.append(cur_list_word)
+                returned_list.append(list_list)
+        return returned_list
+
+    @staticmethod
+    def decode_attrs(attrs: list) -> dict:
+        """ Decodes attrs list and returns a dict with more Word applicable form. """
+        attrs_dict = dict()
+        for attr in attrs:
+            if attr[0] == 'style':
+                attr_list = attr[1].split(':')
+                if attr_list[0] == 'color':
+                    color_hex = attr_list[1].replace('#', '')
+                    r, g, b = str(int(color_hex[0:2], 16)), str(int(color_hex[2:4], 16)), str(int(color_hex[4:6], 16))
+                    attrs_dict['color'] = ','.join([r, g, b])
+                elif attr_list[0] == 'background-color':
+                    attrs_dict['highlight_color'] = attr_list[1]
+                elif attr_list[0] == 'text-align':
+                    attrs_dict['alignment'] = attr_list[1].upper()
+                elif attr_list[0] == 'margin-left':
+                    attrs_dict['margin-left'] = int(attr_list[1].replace('px', ''))
+                elif attr_list[0] == 'margin-right':
+                    attrs_dict['margin-right'] = int(attr_list[1].replace('px', ''))
+        return attrs_dict
+
+    @staticmethod
+    def get_closest_color(hex_str: str) -> str:
+        """ Takes hex color definition. Returns a key (color name) as string.
+            :param str hex_str: Just a string (text) to be added
+        """
+        colors = {(255.0, 255.0, 255.0): 'AUTO', (0.0, 0.0, 0.0): 'BLACK', (0.0, 0.0, 255.0): 'BLUE',
+                  (0.0, 255.0, 0.0): 'BRIGHT_GREEN', (0.0, 0.0, 139.0): 'DARK_BLUE', (139.0, 0.0, 0.0): 'DARK_RED',
+                  (204.0, 204.0, 0.0): 'DARK_YELLOW', (192.0, 192.0, 192.0): 'GRAY_25', (255.0, 0.0, 0.0): 'RED',
+                  (128.01, 128.01, 128.01): 'GRAY_50', (0.0, 127.5, 0.0): 'GREEN', (255.0, 150.0, 180.0): 'PINK',
+                  (0.0, 128.01, 128.01): 'TEAL', (64.0, 224.0, 208.0): 'TURQUOISE', (199.0, 20.0, 133.0): 'VIOLET',
+                  (255.0, 255.0, 255.0): 'WHITE', (255.0, 255.0, 0.0): 'YELLOW'}
+        color_hex = hex_str.replace('#', '')
+        r, g, b = int(color_hex[0:2], 16), int(color_hex[2:4], 16), int(color_hex[4:6], 16)
+        color_diffs = []
+        for color, color_name in colors.items():
+            cr, cg, cb = color
+            color_diff = sqrt(abs(r - cr) ** 2 + abs(g - cg) ** 2 + abs(b - cb) ** 2)
+            color_diffs.append((color_diff, color_name))
+        return min(color_diffs)[1]
+
+
+class ArgHTMLParser(HTMLParser, ABC):
+    """
+    Class based on HTMLParser, used for parsing HTML string passed in get_mapped_html method.
+    Returns a list representing HTML document, which could be further processed to get the nested HTML mapped.
+    """
+    def __init__(self):
+        super().__init__()
+        self.mapped_list = list()
+
+    def handle_starttag(self, tag, attrs):
+        self.mapped_list.append((f'_!{tag}!_', attrs))
+
+    def handle_endtag(self, tag):
+        self.mapped_list.append((f'__!{tag}!__',))
+
+    def handle_data(self, data):
+        if data != ' ' and data != '\n' and data != '\n\n' and data != '\n\t':
+            self.mapped_list.append(('!__data__!', data))
+
+    def get_mapped_html(self, data_str: str) -> list:
+        self.feed(data=data_str)
+        return self.mapped_list
