@@ -1,4 +1,4 @@
-#HEADER
+# HEADER
 #                      arg/Backend/argLaTeXBackend.py
 #               Automatic Report Generator (ARG) v. 1.0
 #
@@ -34,7 +34,7 @@
 #
 # Questions? Visit gitlab.com/AutomaticReportGenerator/arg
 #
-#HEADER
+# HEADER
 
 import numbers
 import os
@@ -44,10 +44,10 @@ import sys
 
 import pylatex as pl
 import yaml
-from pylatex.utils import NoEscape, bold, italic, verbatim
+from pylatex.utils import NoEscape, bold, italic, verbatim, escape_latex
 
 from arg.Common.argMultiFontStringHelper import argMultiFontStringHelper
-from arg.Backend.argBackendBase import argBackendBase
+from arg.Backend.argBackendBase import argBackendBase, ArgHTMLParser
 
 # Load supported colors
 common_dir = os.path.dirname(os.path.realpath(__file__))
@@ -229,12 +229,21 @@ class argLaTeXBackend(argBackendBase):
         # Improve handling of floats
         self.Report.preamble.append(pl.Command("usepackage", "float"))
 
+        # Allow for striked font
+        self.Report.preamble.append(pl.Command("usepackage", "ulem", ["normalem"]))
+
+        # Allow for alignment and justify
+        self.Report.preamble.append(pl.Command("usepackage", "ragged2e"))
+
         # Insert spaces correctly after macros
         self.Report.preamble.append(pl.Command("usepackage", "xspace"))
 
         # Allow for slanted looking fractions
         self.Report.preamble.append(pl.Command("usepackage", "xfrac"))
 
+        # Allow for landscape mode
+        self.Report.preamble.append(pl.Command("usepackage", "pdflscape"))
+
         # Allow for colored table
         self.Report.preamble.append(pl.Command("usepackage", "colortbl"))
 
@@ -250,7 +259,8 @@ class argLaTeXBackend(argBackendBase):
 
         # Allow for hyperlink
         self.Report.preamble.append(pl.Command("usepackage", "hyperref", ["hidelinks"]))
-        self.Report.preamble.append(pl.Command("hypersetup", "colorlinks=true, urlcolor=blue, filecolor=blue, linkcolor=black"))
+        self.Report.preamble.append(
+            pl.Command("hypersetup", "colorlinks=true, urlcolor=blue, filecolor=blue, linkcolor=black"))
 
     def generate_details(self):
         """Generate details: title, organization, address, authors, numbers, etc.
@@ -394,34 +404,112 @@ class argLaTeXBackend(argBackendBase):
         decorated_string = r''
 
         # Iterate over string map
-        for string, font_bits, color in multi_font_string.iterator():
-            # Distinguish between supported bits
-            if font_bits & 16 == 16:
-                # Map Greek letter to unicode when requested
-                string = "${}$".format(
-                    self.GreekLetters.get(string, ''))
+        for string, font_bits, color, highlight_color in multi_font_string.iterator():
+            if isinstance(font_bits, int):
+                if font_bits == 16:
+                    # Map Greek letter to unicode when requested
+                    string = "${}$".format(self.GreekLetters.get(string, ''))
 
-            else:
-                # For all other cases escape LaTeX special characters
-                for c in self.SpecialCharacters:
-                    if c in string:
-                        string = string.replace(c, "\\" + c)
+                else:
+                    # For all other cases escape LaTeX special characters
+                    if font_bits not in [9, 10, 11, 12, 13, 14]:
+                        for c in self.SpecialCharacters:
+                            if c in string:
+                                string = string.replace(c, "\\" + c)
 
-                if font_bits & 1 == 1:
+                    if font_bits == 1:
+                        string = italic(string)
+                    elif font_bits == 2:
+                        string = bold(string)
+                    elif font_bits == 3:
+                        string = r"\underline{{{}}}".format(string)
+                    elif font_bits == 4:
+                        string = r"\texttt{{{}}}".format(string)
+                    elif font_bits == 5:
+                        string = r"\sout{{{}}}".format(string)
+                    elif font_bits == 6:
+                        string = r"\textsubscript{" + f'{string}' + r"}"
+                    elif font_bits == 8:
+                        string = r"$\mathcal{{{}}}$".format(string)
+                    elif font_bits == 9:
+                        string = bold(string)
+                        string = r"{\Huge " + f'{string}' + r"}"
+                    elif font_bits == 10:
+                        string = bold(string)
+                        string = r"{\huge " + f'{string}' + r"}"
+                    elif font_bits == 11:
+                        string = bold(string)
+                        string = r"{\LARGE " + f'{string}' + r"}"
+                    elif font_bits == 12:
+                        string = bold(string)
+                        string = r"{\Large " + f'{string}' + r"}"
+                    elif font_bits == 13:
+                        string = r"{\Large " + f'{string}' + r"}"
+                    elif font_bits == 14:
+                        string = italic(string)
+                        string = r"{\Large " + f'{string}' + r"}"
+
+            elif isinstance(font_bits, list):
+                if 16 in font_bits:
+                    # Map Greek letter to unicode when requested
+                    string = "${}$".format(self.GreekLetters.get(string, ''))
+                else:
+                    if font_bits not in [9, 10, 11, 12, 13, 14]:
+                        for c in self.SpecialCharacters:
+                            if c in string:
+                                string = string.replace(c, "\\" + c)
+                if 1 in font_bits:
                     string = italic(string)
-                if font_bits & 2 == 2:
+                if 2 in font_bits:
                     string = bold(string)
-                if font_bits & 4 == 4:
+                if 3 in font_bits:
+                    string = r"\underline{{{}}}".format(string)
+                if 4 in font_bits:
                     string = r"\texttt{{{}}}".format(string)
-                if font_bits & 8 == 8:
+                if 5 in font_bits:
+                    string = r"\sout{{{}}}".format(string)
+                if 6 in font_bits:
+                    string = r"\textsubscript{" + f'{string}' + r"}"
+                if 8 in font_bits:
                     string = r"$\mathcal{{{}}}$".format(string)
-                if color and self.colors.get(color):
-                    sep = ','
-                    rgb = self.colors.get(color).split(sep)
-                    string = r"\color[rgb]{{{}, {}, {}}}{}".format(int(rgb[0]) / 255.,
-                                                                   int(rgb[1]) / 255.,
-                                                                   int(rgb[2]) / 255.,
-                                                                   string)
+                elif 9 in font_bits:
+                    string = r"{\Huge " + f'{string}' + r"}"
+                elif 10 in font_bits:
+                    string = r"{\huge " + f'{string}' + r"}"
+                elif 11 in font_bits:
+                    string = r"{\LARGE " + f'{string}' + r"}"
+                elif 12 in font_bits:
+                    string = r"{\Large " + f'{string}' + r"}"
+                elif 13 in font_bits:
+                    string = r"{\Large " + f'{string}' + r"}"
+                elif 14 in font_bits:
+                    string = r"{\Large " + f'{string}' + r"}"
+
+            if color:
+                if self.colors.get(color, None) is None:
+                    rgb = color.split(',')
+                    if len(rgb) > 2:
+                        string = r'{\color[rgb]{' + f'{int(rgb[0]) / 255.},' + f'{int(rgb[1]) / 255.},' + \
+                                 f'{int(rgb[2]) / 255.}' + r'}' + f'{string}' + r'}'
+                else:
+                    rgb = self.colors.get(color).split(',')
+                    if len(rgb) > 2:
+                        string = r'{\color[rgb]{' + f'{int(rgb[0]) / 255.},{int(rgb[1]) / 255.},{int(rgb[2]) / 255.}' +\
+                                 r'}' + f'{string}' + r'}'
+
+            if highlight_color is not None:
+                color = self.get_closest_color(hex_str=highlight_color)
+                wd_color_mapping = {'AUTO': '1.0,1.0,1.0', 'BLACK': '0.0,0.0,0.0',
+                                    'BLUE': '0.0,0.0,1.0', 'BRIGHT_GREEN': '0.0,1.0,0.0',
+                                    'DARK_BLUE': '0.0,0.0,0.5451', 'DARK_RED': '0.5451,0.0,0.0',
+                                    'DARK_YELLOW': '0.8,0.8,0.0', 'GRAY_25': '0.753,0.753,0.753',
+                                    'GRAY_50': '0.502,0.502,0.502', 'GREEN': '0.0,0.5,0.0',
+                                    'PINK': '1.0,0.588,0.70588', 'RED': '1.0,0.0,0.0',
+                                    'TEAL': '0.0,0.502,0.502', 'TURQUOISE': '0.251,0.8784,0.8156',
+                                    'VIOLET': '0.7804,0.08235294,0.52156863', 'WHITE': '1.0,1.0,1,0',
+                                    'YELLOW': '1.0,1.0,0.0'}
+                h_rgb = wd_color_mapping.get(color.upper(), 'AUTO').split(',')
+                string = r'\colorbox[rgb]{' + f'{h_rgb[0]},{h_rgb[1]},{h_rgb[2]}' + r'}{' + f'{string}' + r'}'
 
             # Append current decorated string to global one
             decorated_string += string
@@ -478,8 +566,10 @@ class argLaTeXBackend(argBackendBase):
             hyperlink_string = item.get('hyperlink_string', None)
             if "string" in item:
                 self.support_string(item=item)
-                self.Report.append(NoEscape(r"\\"))
             self.Report.append(NoEscape(r"\href{" + hyperlink_path + r"}{\underline{" + hyperlink_string + r"}}"))
+            if "string_suffix" in item:
+                string_suffix = item.get('string_suffix', None)
+                self.Report.append(NoEscape(rf'{string_suffix}'))
             self.Report.append(NoEscape(r"\\"))
 
         # Check whether a string or a file is to be included
@@ -529,6 +619,8 @@ class argLaTeXBackend(argBackendBase):
         # Decorate multi-font strings with LaTeX markup
         if isinstance(title_string, argMultiFontStringHelper):
             title_string = title_string.execute_backend()
+        else:
+            title_string = escape_latex(title_string)
 
         # Add page break if compact paragraph to come
         if item.get("sections"):
@@ -628,9 +720,8 @@ class argLaTeXBackend(argBackendBase):
             row_size = NoEscape(r"\normalsize ")
 
         # Create function to generate decorated strings when needed
-        decorator = (lambda x: NoEscape(x.execute_backend())
-        if isinstance(x, argMultiFontStringHelper)
-        else x)
+        decorator = (
+            lambda x: NoEscape(x.execute_backend()) if isinstance(x, argMultiFontStringHelper) else escape_latex(x))
 
         # Create table
         tab_format = "@{}l" + (n_cols - 1) * 'r' + "@{}"
@@ -698,7 +789,7 @@ class argLaTeXBackend(argBackendBase):
                     # Directly insert base strings
                     table.append(pl.Command(
                         "caption",
-                        NoEscape(caption_string)))
+                        escape_latex(caption_string)))
 
                 elif isinstance(caption_string, argMultiFontStringHelper):
                     # Insert LaTeX string created from multi-font string
@@ -733,7 +824,7 @@ class argLaTeXBackend(argBackendBase):
             row_size = NoEscape(r"\normalsize")
 
         # Create function to generate decorated strings when needed
-        dec = lambda x: NoEscape(x.execute_backend()) if isinstance(x, argMultiFontStringHelper) else x
+        dec = lambda x: NoEscape(x.execute_backend()) if isinstance(x, argMultiFontStringHelper) else escape_latex(x)
 
         # Create table
         tab_format = self.get_table_format(headers=headers)
@@ -751,7 +842,7 @@ class argLaTeXBackend(argBackendBase):
             table.append(NoEscape(r"\toprule"))
             header_list = [NoEscape(
                 rf"{color_decoder(rgb_color=x[2], text='color')}{row_size}{color_decoder(rgb_color=x[1])}{dec(x[0])}")
-                           for x in headers]
+                for x in headers]
             table.add_row(header_list)
             table.append(NoEscape(r"\midrule\endfirsthead"))
 
@@ -759,7 +850,7 @@ class argLaTeXBackend(argBackendBase):
             for row in rows:
                 table.add_row([NoEscape(
                     rf"{color_decoder(rgb_color=x[2], text='color')}{row_size}{color_decoder(rgb_color=x[1])}{dec(x[0])}")
-                               for x in row])
+                    for x in row])
 
             # Create table footer
             table.append(NoEscape(r"\bottomrule\\"))
@@ -768,12 +859,56 @@ class argLaTeXBackend(argBackendBase):
             if caption is not None:
                 if isinstance(caption, str):
                     # Directly insert base strings
-                    table.append(pl.Command("caption", NoEscape(caption)))
+                    table.append(pl.Command("caption", escape_latex(caption)))
 
                 elif isinstance(caption, argMultiFontStringHelper):
                     # Insert LaTeX string created from multi-font string
                     table.append(pl.Command("caption", NoEscape(caption.execute_backend())))
 
+    def add_html(self, data: dict) -> None:
+        """ Adds html content to pdf document. """
+        # Reading html data as a string
+        html_str = data.get('html_string', None)
+        # Parsing html data and get a html document in a form of list
+        arg_html_parser = ArgHTMLParser()
+        arg_html_parser.reset()
+        html_list = arg_html_parser.get_mapped_html(html_str)
+        # Parsing an html list into nested list over which is easier to iterate
+        nested_list = self.nesting_html_list(html_list=html_list)
+        # Parsing nested list to a list of mainly ARG Multi Font String Helper, ready to put in directly into document
+        amfsh_list = self.map_nested_list_to_amfsh(nested_list=nested_list)
+        # Final iteration over ARG Multi Font String Helper list
+        for amfsh in amfsh_list:
+            alingment_map = {'LEFT': [NoEscape(r"\begin{FlushLeft}"), NoEscape(r"\end{FlushLeft}")],
+                             'RIGHT': [NoEscape(r"\begin{FlushRight}"), NoEscape(r"\end{FlushRight}")],
+                             'CENTER': [NoEscape(r"\begin{Center}"), NoEscape(r"\end{Center}")],
+                             'JUSTIFY': [NoEscape(r"\begin{justify}"), NoEscape(r"\end{justify}")],
+                             }
+            # Checking for tuple. Tuple instance is dedicated for any element except ordered and unordered lists
+            if isinstance(amfsh, tuple):
+                # amfsh as tuple: (alignment, argMultiFontStringHelper, margin)
+                self.Report.append(pl.Command("par"))
+                self.Report.append(alingment_map.get(amfsh[0], 'LEFT')[0])
+                string = self.generate_multi_font_string(multi_font_string=amfsh[1])
+                self.Report.append(NoEscape(string))
+                self.Report.append(alingment_map.get(amfsh[0], 'LEFT')[1])
+            # Checking for list. List instance is dedicated for ordered and unordered lists (needs paragraph styling)
+            elif isinstance(amfsh, list):
+                # amfsh as list: [un/ordered list element, un/ordered list element, un/ordered list element, ...]
+                lst_type = {'List Bullet': 'itemize', 'List Number': 'enumerate'}
+                list_end = len(amfsh)
+                list_type = lst_type.get(amfsh[0][3])
+                for num, elem in enumerate(amfsh, start=1):
+                    if num == 1:
+                        lst_command = r"\begin{" + f"{list_type}" + r"}"
+                        self.Report.append(NoEscape(lst_command))
+                    string = self.generate_multi_font_string(multi_font_string=elem[1])
+                    string = r"\item " + string
+                    self.Report.append(NoEscape(string))
+                    if num == list_end:
+                        lst_command = r"\end{" + f"{list_type}" + r"}"
+                        self.Report.append(NoEscape(lst_command))
+
     def add_figure(self, arguments):
         """Add figure to the report
         """
@@ -786,9 +921,15 @@ class argLaTeXBackend(argBackendBase):
             return
 
         # Retrieve image and associated caption for figure creation
-        figure_file_name, caption_string = self.fetch_image_and_caption(
+        figure_file_name, caption_string, from_file = self.fetch_image_and_caption(
             arguments)
 
+        # The caption_string is already escaped
+        if from_file is not None:
+            already_escaped = True
+        else:
+            already_escaped = None
+
         # Create figure when available
         if figure_file_name:
             with self.Report.create(pl.Figure(position="ht!")) as picture:
@@ -796,12 +937,19 @@ class argLaTeXBackend(argBackendBase):
 
                 # Append caption when available
                 if caption_string:
-                    if isinstance(caption_string, argMultiFontStringHelper):
+                    if already_escaped is not None and not isinstance(caption_string, argMultiFontStringHelper):
+                        caption_string = NoEscape(caption_string)
+
+                    elif isinstance(caption_string, str):
+                        # Directly insert base strings
+                        caption_string = escape_latex(caption_string)
+
+                    elif isinstance(caption_string, argMultiFontStringHelper):
                         # Convert multi-font strings to LaTeX
-                        caption_string = caption_string.execute_backend()
+                        caption_string = NoEscape(caption_string.execute_backend())
 
                     # Insert caption
-                    picture.add_caption(NoEscape(caption_string))
+                    picture.add_caption(caption_string)
 
                 # Insert label when available
                 figure_label = arguments.get("label")
@@ -889,10 +1037,10 @@ class argLaTeXBackend(argBackendBase):
                 # Append color-table
                 self.add_color_table(data=item)
 
-            # Handle hyperlink
-            elif item_type == "hyperlink":
-                # Append a hyperlink
-                self.add_hyperlink(data=item)
+            # Handle html
+            elif item_type == "html":
+                # Append html
+                self.add_html(data=item)
 
             # Proceed with recursion if needed
             if "sections" in item:
@@ -982,6 +1130,8 @@ class argLaTeXBackend(argBackendBase):
         # Decorate multi-font strings with LaTeX markup
         if isinstance(string, argMultiFontStringHelper):
             string = self.generate_multi_font_string(string)
+        else:
+            string = self.apply_break_lines(no_lb_string=string)
 
         # Insert string into the report
         self.Report.append(NoEscape(string))
@@ -1002,3 +1152,9 @@ class argLaTeXBackend(argBackendBase):
         hor_al.append('@{}')
         return ''.join(hor_al)
 
+    @staticmethod
+    def apply_break_lines(no_lb_string: str) -> str:
+        """ Takes string without break lines"""
+        lb_string1 = no_lb_string.replace('\\', r'\\')
+        lb_string = lb_string1.replace('\n', r'\\')
+        return lb_string
