@@ -35,7 +35,7 @@
 # Questions? Visit gitlab.com/AutomaticReportGenerator/arg
 #
 # HEADER
-
+from io import BytesIO
 import numbers
 import os
 import platform
@@ -47,13 +47,17 @@ import yaml
 import docx
 from docx.opc.constants import RELATIONSHIP_TYPE as RT
 from docx.enum.text import WD_PARAGRAPH_ALIGNMENT as WD_ALIGN_PARAGRAPH
+from docx.text.paragraph import Paragraph
+from docx.enum.text import WD_COLOR_INDEX
+from docx.enum.section import WD_ORIENTATION, WD_SECTION_START
+from docx.enum.shape import WD_INLINE_SHAPE
 from docx.oxml import OxmlElement, parse_xml
 from docx.oxml.ns import qn, nsdecls
 from docx.shared import Mm, Cm, Inches, RGBColor, Pt
 from pylatex import NoEscape
 
 from arg.Common.argMultiFontStringHelper import argMultiFontStringHelper
-from arg.Backend.argBackendBase import argBackendBase
+from arg.Backend.argBackendBase import argBackendBase, ArgHTMLParser
 
 osn = os.name
 
@@ -192,7 +196,7 @@ class argWordBackend(argBackendBase):
         # Generate a backend dependent text
         return "{}".format(text)
 
-    def generate_multi_font_string(self, multi_font_string, paragraph=None):
+    def generate_multi_font_string(self, multi_font_string: argMultiFontStringHelper, paragraph=None):
         """Either append Word runs to paragraph or return text string
         """
 
@@ -200,26 +204,101 @@ class argWordBackend(argBackendBase):
         if paragraph:
 
             # Iterate over all items in string helper
-            for string, font_bits, color in multi_font_string.iterator():
+            for string, font_bits, color, highlight_color in multi_font_string.iterator():
 
-                # Map Greek letter to unicode when requested
-                if font_bits & 16 == 16:
-                    string = self.GreekLetters.get(string, '')
+                if isinstance(font_bits, int):
+                    # Map Greek letter to unicode when requested
+                    if font_bits == 16:
+                        string = self.GreekLetters.get(string, '')
+
+                    # Add a run per string and set font per defined bits
+                    run = paragraph.add_run(string)
+
+                    if font_bits == 1:
+                        run.font.italic = True
+                    elif font_bits == 2:
+                        run.font.bold = True
+                    elif font_bits == 3:
+                        run.font.underline = True
+                    elif font_bits == 4:
+                        run.font.name = self.fonts.get("typewriter", "default" if self.fonts.get("default") else None)
+                    elif font_bits == 5:
+                        run.font.strike = True
+                    elif font_bits == 6:
+                        run.font.subscript = True
+                    elif font_bits == 8:
+                        run.font.name = self.fonts.get("calligraphic", "default" if self.fonts.get("default") else None)
+                    elif font_bits == 9:
+                        run.font.size = Pt(14)
+                        run.font.bold = True
+                    elif font_bits == 10:
+                        run.font.size = Pt(13)
+                        run.font.bold = True
+                    elif font_bits == 11:
+                        run.font.size = Pt(11)
+                        run.font.bold = True
+                    elif font_bits == 12:
+                        run.font.size = Pt(11)
+                        run.font.bold = True
+                        run.font.italic = True
+                    elif font_bits == 13:
+                        run.font.size = Pt(11)
+                    elif font_bits == 14:
+                        run.font.size = Pt(11)
+                        run.font.italic = True
+                    else:
+                        run.font.name = self.fonts.get("default", None)
+
+                elif isinstance(font_bits, list):
+                    if 16 in font_bits:
+                        string = self.GreekLetters.get(string, '')
+                    run = paragraph.add_run(string)
+                    run.font.italic = True if 1 in font_bits else False
+                    run.font.bold = True if 2 in font_bits else False
+                    run.font.underline = True if 3 in font_bits else False
+                    run.font.strike = True if 5 in font_bits else False
+                    run.font.subscript = True if 6 in font_bits else False
+                    if 4 in font_bits:
+                        run.font.name = self.fonts.get("typewriter", "default" if self.fonts.get("default") else None)
+                    elif 8 in font_bits:
+                        run.font.name = self.fonts.get("calligraphic", "default" if self.fonts.get("default") else None)
+                    elif 9 in font_bits:
+                        run.font.size = Pt(14)
+                    elif 10 in font_bits:
+                        run.font.size = Pt(13)
+                    elif 11 in font_bits:
+                        run.font.size = Pt(11)
+                    elif 12 in font_bits:
+                        run.font.size = Pt(11)
+                    elif 13 in font_bits:
+                        run.font.size = Pt(11)
+                    elif 14 in font_bits:
+                        run.font.size = Pt(11)
+                    else:
+                        run.font.name = self.fonts.get("default", None)
 
-                # Add a run per string and set font per defined bits
-                run = paragraph.add_run(string)
-                run.italic = font_bits & 1 == 1
-                run.bold = font_bits & 2 == 2
-                if font_bits & 4 == 4:
-                    run.font.name = self.fonts.get("typewriter", "default" if self.fonts.get("default") else None)
-                elif font_bits & 8 == 8:
-                    run.font.name = self.fonts.get("calligraphic", "default" if self.fonts.get("default") else None)
-                elif self.fonts.get("default", None):
-                    run.font.name = self.fonts.get("default", None)
                 if color:
-                    rgb = self.colors.get(color).split(',')
-                    if len(rgb) > 2:
-                        run.font.color.rgb = RGBColor(*[int(x) for x in rgb[:3]])
+                    if self.colors.get(color, None) is None:
+                        rgb = color.split(',')
+                        if len(rgb) > 2:
+                            run.font.color.rgb = RGBColor(*[int(x) for x in rgb[:3]])
+                    else:
+                        rgb = self.colors.get(color).split(',')
+                        if len(rgb) > 2:
+                            run.font.color.rgb = RGBColor(*[int(x) for x in rgb[:3]])
+
+                if highlight_color is not None:
+                    color = self.get_closest_color(hex_str=highlight_color)
+                    wd_color_mapping = {'AUTO': WD_COLOR_INDEX.AUTO, 'BLACK': WD_COLOR_INDEX.BLACK,
+                                        'BLUE': WD_COLOR_INDEX.BLUE, 'BRIGHT_GREEN': WD_COLOR_INDEX.BRIGHT_GREEN,
+                                        'DARK_BLUE': WD_COLOR_INDEX.DARK_BLUE, 'DARK_RED': WD_COLOR_INDEX.DARK_RED,
+                                        'DARK_YELLOW': WD_COLOR_INDEX.DARK_YELLOW, 'GRAY_25': WD_COLOR_INDEX.GRAY_25,
+                                        'GRAY_50': WD_COLOR_INDEX.GRAY_50, 'GREEN': WD_COLOR_INDEX.GREEN,
+                                        'PINK': WD_COLOR_INDEX.PINK, 'RED': WD_COLOR_INDEX.RED,
+                                        'TEAL': WD_COLOR_INDEX.TEAL, 'TURQUOISE': WD_COLOR_INDEX.TURQUOISE,
+                                        'VIOLET': WD_COLOR_INDEX.VIOLET, 'WHITE': WD_COLOR_INDEX.WHITE,
+                                        'YELLOW': WD_COLOR_INDEX.YELLOW}
+                    run.font.highlight_color = wd_color_mapping.get(color.upper(), 'AUTO')
 
             # Nothing is returned in this case
             return
@@ -306,11 +385,12 @@ class argWordBackend(argBackendBase):
         if "hyperlink_path" in item:
             hyperlink_path = item.get('hyperlink_path', None)
             hyperlink_string = item.get('hyperlink_string', None)
-            if "string" in item:
-                self.support_string(item=item)
-            par = p if p is not None else self.Report.add_paragraph()
-            self.add_hyperlink_to_paragraph(paragraph=par, url=hyperlink_path, text=hyperlink_string, color='0000FF',
-                                            underline=True)
+            par = self.support_string(item=item) if "string" in item else self.Report.add_paragraph()
+            _, par_hyp = self.add_hyperlink_to_paragraph(paragraph=par, url=hyperlink_path, text=hyperlink_string,
+                                                         color='0000FF', underline=True)
+            if "string_suffix" in item:
+                string_suffix = item.get('string_suffix', None)
+                par_str_suff = self.add_string_to_paragraph(paragraph=par_hyp, text=string_suffix)
 
         # Check whether a string or a file is to be included
         elif "string" in item:
@@ -581,10 +661,20 @@ class argWordBackend(argBackendBase):
             # Insert blank paragraph to ensure table is closed
             self.Report.add_paragraph()
 
+    def change_orientation(self):
+        """ Change orientation of the page from portrait to landscape or the other way around when called
+        """
+        current_section = self.Report.sections[-1]
+        new_width, new_height = current_section.page_height, current_section.page_width
+        new_section = self.Report.add_section(WD_SECTION_START.NEW_PAGE)
+        new_section.orientation = WD_ORIENTATION.LANDSCAPE
+        new_section.page_width = new_width
+        new_section.page_height = new_height
+        return new_section
+
     def add_color_table(self, data: dict) -> None:
         """ Add colored table to the report
         """
-
         headers_list = data.get('headers', None)
         table_columns_num = len(headers_list)
         rows_list = data.get('rows', None)
@@ -669,6 +759,116 @@ class argWordBackend(argBackendBase):
             # Insert blank paragraph to ensure table is closed
             self.Report.add_paragraph()
 
+    def inline_docx(self, data: dict) -> None:
+        """ Add external docx document to the report
+        """
+        docx_path = data.get('docx_path', None)
+        docx_path = os.path.join(self.Parameters.DataDir, docx_path)
+        if docx_path is None:
+            raise FileNotFoundError('Path to file NOT found!')
+        elif not os.path.isfile(docx_path):
+            raise FileNotFoundError('File does NOT exists!')
+        else:
+            # Mark in which paragraph there is an image
+            inline_docx = docx.Document(docx_path)
+            image_in_par = [True if 'graphicData' in par._p.xml else False for par in inline_docx.paragraphs]
+
+            inline_docx = docx.Document(docx_path)
+            # Setting counter coresponding to paragraphs
+            counter = 0
+            # Iterating over elements in document to inline
+            for element in inline_docx.element.body:
+                # If element is a paragraph, counter gets incremented
+                if isinstance(element, docx.oxml.text.paragraph.CT_P):
+                    # Eliminate use of style which does not exist
+                    if element.style == 'Caption1':
+                        element.style = 'Caption'
+                    # Checking if paragraph had an image
+                    if image_in_par[counter]:
+                        # Taking an image from inline document and putting it to the last paragraph from current report
+                        inl_docx = docx.Document(docx_path)
+                        for shape in inl_docx.inline_shapes:
+                            if shape.type == WD_INLINE_SHAPE.PICTURE:
+                                # Make sure image isn't wider than 159mm, if it is, then set width do 159mm
+                                if shape.width.mm > 159:
+                                    ratio = shape.width.mm / 159
+                                    pic_width = shape.width / ratio
+                                else:
+                                    pic_width = shape.width
+                                # Taking image from inlining document
+                                inline = shape._inline
+                                rId = inline.xpath('./a:graphic/a:graphicData/pic:pic/pic:blipFill/a:blip/@r:embed')[0]
+                                image_part = inl_docx._part.related_parts[rId]
+                                image_bytes = image_part.blob
+                                # write the image bytes to a file (or BytesIO stream)
+                                image_stream = BytesIO(image_bytes)
+                                last_par = self.Report.paragraphs[-1]
+                                new_par = self.insert_paragraph_after(paragraph=last_par)
+                                r = new_par.add_run()
+                                # finally adding a picture with correct width
+                                r.add_picture(image_stream, width=pic_width)
+                        counter += 1
+                    # Paragraph with no photo is simply added to the current report
+                    else:
+                        self.Report.element.body.append(element)
+                        counter += 1
+                # If element was not a paragraph is simply added to the current report
+                else:
+                    self.Report.element.body.append(element)
+
+    def add_html(self, data: dict) -> None:
+        """ Adds html content to Word document. """
+        # Reading html data as a string
+        html_str = data.get('html_string', None)
+        # Parsing html data and get a html document in a form of list
+        arg_html_parser = ArgHTMLParser()
+        arg_html_parser.reset()
+        html_list = arg_html_parser.get_mapped_html(html_str)
+        # Parsing an html list into nested list over which is easier to iterate
+        nested_list = self.nesting_html_list(html_list=html_list)
+        # Parsing nested list to a list of mainly ARG Multi Font String Helper, ready to put in directly into document
+        amfsh_list = self.map_nested_list_to_amfsh(nested_list=nested_list)
+        # Final iteration over ARG Multi Font String Helper list
+        for amfsh in amfsh_list:
+            # Checking for tuple. Tuple instance is dedicated for any element except ordered and unordered lists
+            if isinstance(amfsh, tuple):
+                # amfsh as tuple: (alignment, argMultiFontStringHelper, margin)
+                new_par = self.Report.add_paragraph()
+                algn_map = {'CENTER': WD_ALIGN_PARAGRAPH.CENTER, 'LEFT': WD_ALIGN_PARAGRAPH.LEFT,
+                            'RIGHT': WD_ALIGN_PARAGRAPH.RIGHT, 'JUSTIFY': WD_ALIGN_PARAGRAPH.JUSTIFY}
+                new_par.alignment = algn_map.get(amfsh[0], WD_ALIGN_PARAGRAPH.LEFT)
+                if amfsh[2] is not None and amfsh[2][0] == 'margin-right':
+                    new_par.paragraph_format.right_indent = Pt(amfsh[2][1])
+                elif amfsh[2] is not None and amfsh[2][0] == 'margin-left':
+                    new_par.paragraph_format.left_indent = Pt(amfsh[2][1])
+                self.generate_multi_font_string(multi_font_string=amfsh[1], paragraph=new_par)
+            # Checking for list. List instance is dedicated for ordered and unordered lists (needs paragraph styling)
+            elif isinstance(amfsh, list):
+                # amfsh as list: [un/ordered list element, un/ordered list element, un/ordered list element, ...]
+                for elem in amfsh:
+                    # elem as list: [alignment, argMultiFontStringHelper, margin, list style]
+                    new_par = self.Report.add_paragraph(style=elem[3])
+                    algn_map = {'CENTER': WD_ALIGN_PARAGRAPH.CENTER, 'LEFT': WD_ALIGN_PARAGRAPH.LEFT,
+                                'RIGHT': WD_ALIGN_PARAGRAPH.RIGHT, 'JUSTIFY': WD_ALIGN_PARAGRAPH.JUSTIFY}
+                    new_par.alignment = algn_map.get(elem[0], WD_ALIGN_PARAGRAPH.LEFT)
+                    if elem[2] is not None and elem[2][0] == 'margin-right':
+                        new_par.paragraph_format.right_indent = Pt(elem[2][1])
+                    elif elem[2] is not None and elem[2][0] == 'margin-left':
+                        new_par.paragraph_format.left_indent = Pt(elem[2][1])
+                    self.generate_multi_font_string(multi_font_string=elem[1], paragraph=new_par)
+
+    @staticmethod
+    def insert_paragraph_after(paragraph, text=None, style=None):
+        """Insert a new paragraph after the given paragraph."""
+        new_p = OxmlElement("w:p")
+        paragraph._p.addnext(new_p)
+        new_para = Paragraph(new_p, paragraph._parent)
+        if text:
+            new_para.add_run(text)
+        if style is not None:
+            new_para.style = style
+        return new_para
+
     def add_hyperlink(self, data: dict) -> None:
         """ Add hyperlink to the report
         """
@@ -855,6 +1055,16 @@ class argWordBackend(argBackendBase):
                 # Append color-table
                 self.add_color_table(data=item)
 
+            # Handle inline document
+            elif item_type == "inline-docx":
+                # Append inline-docx
+                self.inline_docx(data=item)
+
+            # Handle html
+            elif item_type == "html":
+                # Append html
+                self.add_html(data=item)
+
             # Proceed with recursion if needed
             if "sections" in item:
                 self.recursively_build_report(item["sections"], list_lvl)
@@ -877,7 +1087,7 @@ class argWordBackend(argBackendBase):
         self.append_document_postamble()
 
         # Handle title numbering
-        self.add_header_numbering()
+        # self.add_header_numbering()
 
         # Create Word report
         print("[argWordBackend] Saving Word report")
@@ -1117,7 +1327,7 @@ class argWordBackend(argBackendBase):
         table_colors_hex = [[self.parse_rgb_to_hex(color, ct) for color in color_row] for color_row in table_colors]
         return table_colors_hex
 
-    def support_string(self, item: dict) -> None:
+    def support_string(self, item: dict) -> Paragraph:
         """ Function created in order not to DRY
         """
         # Retrieve provided string
@@ -1126,11 +1336,16 @@ class argWordBackend(argBackendBase):
         # Distinguish between plain and multi-font strings
         if isinstance(string, argMultiFontStringHelper):
             # Insert string into the report
-            self.generate_multi_font_string(
-                string, self.Report.add_paragraph())
+            return self.generate_multi_font_string(string, self.Report.add_paragraph())
         else:
             # Directly insert Word fragment into the report
-            self.Report.add_paragraph(string)
+            return self.Report.add_paragraph(self.apply_break_lines(no_lb_string=string))
+
+    @staticmethod
+    def apply_break_lines(no_lb_string: str) -> str:
+        """ Takes string without break lines"""
+        lb_string = no_lb_string.replace('\\', '\n')
+        return lb_string
 
     @staticmethod
     def parse_vertical_alignment(headers: list, rows: list) -> list:
@@ -1212,4 +1427,29 @@ class argWordBackend(argBackendBase):
 
         paragraph._p.append(hyperlink)
 
-        return hyperlink
+        return hyperlink, paragraph
+
+    @staticmethod
+    def add_string_to_paragraph(paragraph, text) -> Paragraph:
+        """
+        A function that places a hyperlink within a paragraph object.
+        :param paragraph: The paragraph we are adding the hyperlink to.
+        :param text: The text displayed for the url
+        :return: The hyperlink object
+        """
+
+        # This gets access to the document.xml.rels file and gets a new relation id value
+        part = paragraph.part
+
+        # Create a w:r element
+        new_run = docx.oxml.shared.OxmlElement('w:r')
+
+        # Create a new w:rPr element
+        rPr = docx.oxml.shared.OxmlElement('w:rPr')
+
+        # Join all the xml elements together add add the required text to the w:r element
+        new_run.append(rPr)
+        new_run.text = text
+        paragraph._p.append(new_run)
+
+        return paragraph
