@@ -49,39 +49,21 @@ from arg.Backend.argBackend import argBackend
 from arg.DataInterface.argDataInterface import argDataInterface
 from arg.Tools import Utilities
 
-app = "argReportParameters"
-
 
 class argReportParameters:
     """A class to get and store ARG report parameters
     """
-
-    # Load valid types
-    local_dir = os.path.dirname(os.path.realpath(__file__))
-    with open(os.path.join(local_dir, "../Common/argTypes.yml"),
-              'r') as t_file:
-        Types = yaml.safe_load(t_file)
-
-    # Retrieve default parameters file value
-    DEFAULT_PARAMETERSFILE = Types.get(
-        "DefaultParametersFile")
-
-    # Verbosity levels
-    VerbosityLevels = Types.get("VerbosityLevels", {})
-    print("[{}] Supported verbosity levels: {}".format(
-        app,
-        ", ".join(VerbosityLevels)))
-
-    # Save default temp file name
-    DEFAULT_TMPFILE = "ARG"
-
     def __init__(self, application, parameters_file=None, version=None, types=None, latex_processor=None, structure_file=None):
 
+        self.DEFAULT_PARAMETERSFILE = None
+        self.VerbosityLevels = None
+
+        self.__init_params()
+
         # Default application settings
         self.Application = application
         self.Version = version
         self.Types = types
-        self.TmpFile = None
         self.LatexProcessor = latex_processor
         self.TexFile = None
 
@@ -146,6 +128,22 @@ class argReportParameters:
         if self.ParametersFile:
             self.parse_parameters_file()
 
+    def __init_params(self):
+        # Load valid types
+        local_dir = os.path.dirname(os.path.realpath(__file__))
+        with open(os.path.join(local_dir, "../Common/argTypes.yml"),
+                  'r') as t_file:
+            Types = yaml.safe_load(t_file)
+
+        # Retrieve default parameters file value
+        self.DEFAULT_PARAMETERSFILE = Types.get(
+            "DefaultParametersFile")
+
+        # Verbosity levels
+        self.VerbosityLevels = Types.get("VerbosityLevels", {})
+        print("[argReportParameters] Supported verbosity levels: {}".format(
+            ", ".join(self.VerbosityLevels)))
+
     def command_line_usage(self):
         """Provide online help on ARG with command line arguments
         """
@@ -254,21 +252,14 @@ class argReportParameters:
         """Parse command line
         """
 
-        parsed = True
-
         # Try parsing command line arguments or get parameters file
+        parsed = True
         if self.Application.capitalize() == "Explorator" and not self.parse_command_line(
                 'Explorator') and not self.parse_parameters_line():
             parsed = False
-        if self.Application.capitalize() == "Generator" and not self.parse_command_line(
-                'Generator') and not self.parse_parameters_line():
-            parsed = False
-        if self.Application.capitalize() == "Assembler" and not self.parse_command_line(
-                'Assembler') and not self.parse_parameters_line():
-            parsed = False
         if self.Application.capitalize() == "ARG":
-            if not (self.parse_command_line('Explorator') and self.parse_command_line('Generator')
-                    and self.parse_command_line('Assembler')):
+            if not (self.parse_command_line("Explorator") and self.parse_command_line("Generator")
+                    and self.parse_command_line("Assembler")):
                 if not self.parse_parameters_line():
                     parsed = False
 
@@ -280,7 +271,7 @@ class argReportParameters:
             print("\nUse a parameters file :")
             self.parameters_file_usage()
 
-        if not (self.Application.capitalize() == "Generator" or self.ParametersFile):
+        if not self.ParametersFile:
             print("*  ERROR: No parameters file provided. Exiting. ")
             sys.exit(1)
 
@@ -1045,39 +1036,3 @@ class argReportParameters:
 
         # Specified file was found
         return True
-
-    def log_execution_status(self, app, tmp_file=None):
-        """ Log execution status to tmp file
-        """
-
-        # Use default tmp file name if not explicitly specified
-        self.TmpFile = "{}.tmp".format(tmp_file or self.DEFAULT_TMPFILE)
-
-        # Write app name to tmp file
-        f = open(self.TmpFile, "a")
-        f.write("{}\n".format(app))
-        f.close()
-
-    def get_successful_apps(self, app):
-        """ Retrieve successful app names from parameters TmpFile attribute
-        """
-
-        # Check if tmp file has been created
-        if self.TmpFile is not None:
-            if os.path.exists(self.TmpFile):
-                # Open tmp file and read content
-                with open(self.TmpFile, 'r') as f:
-                    apps = f.readlines()
-                    res = ', '.join(apps).replace('\n', '')
-                f.close()
-
-                # Remove tmp file
-                os.remove(self.TmpFile)
-
-                # Return formatted apps
-                return res
-
-        # Print error message otherwise
-        else:
-            print("[{}] No tmp file. ".format(
-                app))
